#########################################
# Function for Markov Matrix Estimation #
#########################################

tranMatrixWeightedObs <- function(act,fut,wei,statesAct,statesFut,numStates,TM=1,
                                  EST=1) {

#Function Markov Matrix Estimation of weighted observations and other statistics
#To call remind first: source("tranMatrixWeightedObs.r")

#act:   actual value
#fut:   future value
#wei:   weights of observations
#statesAct: definition of states of actual values
#statesFut: definition of states of future values
#TM=0:  no estimation of transition matrix (useful if there exists a class 
#       without observation)
#EST=0  no estimation of final distribution

##########
# Output #
##########
#transM			: transition matrix
#ergoM			: ergodic matrix
#markovM		: markov matrix
#ergoDistr  : ergodic distribution
#distrObsAct		: initial distribution of observations
#distrObsFut		: final distribution of observations
#numObsClassAct : number of observations for actual states
#numObsClassFut : number of observations for future states
#estDistr		: estimation of final distribution

#Dimensions of samples
ii <-length(act)

#Bulding up matrix for estimation
matrixCurrent <- act
matrixFuture <- fut

#Inizialization markov matrix
markovMatrix <- matrix(0,numStates,numStates)
markovMatrixOBS <- matrix(0,numStates,numStates)


iii <-0

for (i in 1:numStates) {

    iii <-iii+1
    selectObs <- matrixCurrent>=statesAct[i] & matrixCurrent<statesAct[i+1]

    if (sum(selectObs)>0) {

    	sel <- matrixFuture[selectObs]
    	selWei <- wei[selectObs]
    	
			jjj <-0
     	for (k in 1:numStates) {

    			jjj <-jjj+1
     			markovMatrix[iii,jjj] <- sum(selWei[sel>=statesFut[k] & sel<statesFut[k+1]])
     			markovMatrixOBS[iii,jjj] <- sum(selWei[sel>=statesFut[k] & sel<statesFut[k+1]]>0)
     			

					}
		  }

      else {
		    markovMatrix[iii,] <- 0
			  if (TM==1) print("So' cazzi con 'ste classi...cambiale!")
     }
	}
if (TM==1)  {

  #Inizialization of matrices
    transMatrix <- matrix(0,numStates,numStates)
    ergoDistr <- matrix(0,1,numStates)

  #Transition matrix
  	for (i in 1:dim(markovMatrix)[1]) {
  		transMatrix[i,] <-markovMatrix[i,]/sum(markovMatrix[i,])
  	}

  #Ergodic matrix
  	ergoMatrix <- transMatrix
  	for (i in 1:1000) {
  		ergoMatrix <-ergoMatrix%*%transMatrix
  	}
  	ergoDistr <-ergoMatrix[1,]
}
else {
	ergoMatrix <- transMatrix
	ergoDistr <-ergoMatrix[1,]
	print("ATTENTION: transition matrix and ergodic distribution not calculated")
}

#Distribution of observations into actual states
  distrObsAct <- rowSums(markovMatrix)
 	distrObsActDistr <- distrObsAct/sum(distrObsAct)

#Distribution of observations into future states
  distrObsFut <- colSums(markovMatrix)
  distrObsFutDistr <- distrObsFut/sum(distrObsFut)

#Estimation of the final distribution by transition matrix, 
#given initial distribution for the check of estimate
  estDistr <- matrix(0,1,numStates)
    	estDistr <- distrObsActDistr
  	    	estDistr <- estDistr%*%transMatrix
 
#Return of function
  list(transM=transMatrix,ergoM=ergoMatrix,markovM=markovMatrix,
    ergoDistr=ergoDistr,distrObsActDistr=distrObsActDistr,
    distrObsFutDistr=distrObsFutDistr,distrObsAct=distrObsAct,
    distrObsFut=distrObsFut,markovMatrixOBS=markovMatrixOBS,estDistr=estDistr)

}
